import numpy as np
import pickle
import time
import copy
import sys

# ROS Libraries
import rospy
from std_msgs.msg import Bool
from trajectory_msgs.msg import (
    JointTrajectoryPoint,
    JointTrajectory
)

## Our libraries
from utils import load_plan, JTraj2ROS, speed_multiplier
from Controller import Controller
import gcode2contour as gc


def execute_plan(plan,
        controller,
        contours=None,
        confirm=False,
        skip_first=1,
        contour_speed=0.2,
        move_speed=1,
        n_trajs=None):
    """
    Executes a plan on MoveIt

    Single arm only
    skips the first n trajectories in the plan
    """
    # plan to the start position of the plan
    controller.group.go(plan.trajs[0][skip_first].positions[0])

    # execute trajectories in plan
    for i, t in enumerate(plan.trajs[0]):
        print("---")

        if i == n_trajs + skip_first:
            print("Finishing prematurely, completed {} trajectories".format(i - skip_first))
            break

        if i < skip_first:
            print("skipping traj {}".format(i))
            continue

        if confirm:
            a = raw_input('Press enter to continue')
            if a== 'n':
                break

        print("Printing {}th trajectory in the plan, which is contour {}".format(i, t.contour))

        if t.contour is not None and contours is not None:
            while not controller.exec_ctraj(speed_multiplier(t, 1/contour_speed), contour=contours[t.contour]):
                pass
        else:
            #print("Moving arm without extrusion")
            while not controller.exec_ctraj(speed_multiplier(t, 1/move_speed)):
                pass
    return


if __name__ == '__main__':
    plan1 = load_plan('origin')
    calibration_test = load_plan('cal_test')
    plan2, flexirex_contours = load_plan('flexirex', contours=True)

    # Printing
    con = Controller()
    execute_plan(plan2, con, contours=flexirex_contours, confirm=True, n_trajs=10)

    # no printing
    #con = Controller(disable_extruder=True)
    #execute_plan(plan2, con, confirm=False, n_trajs=200, move_speed=1)

    # go to origin and stay (to check calibration)
    #plan05 = copy.deepcopy(plan1)
    #plan05.trajs[0].pop()
    #execute_plan(plan05, con)
